@@ -8,6 +8,7 @@ from . import service
 from . import subscription
 from . import invoice
 from . import party
+from . import ir
 
 
 def register():
@@ -23,12 +24,13 @@ def register():
         subscription.CreateLineConsumptionStart,
         subscription.CreateSubscriptionInvoiceStart,
         invoice.InvoiceLine,
+        ir.Cron,
         module='sale_subscription', type_='model')
     Pool.register(
         subscription.CreateLineConsumption,
         subscription.CreateSubscriptionInvoice,
-        party.PartyReplace,
-        party.PartyErase,
+        party.Replace,
+        party.Erase,
         module='sale_subscription', type_='wizard')
     Pool.register(
         module='sale_subscription', type_='report')
