@@ -1,33 +1,27 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import datetime
-from decimal import Decimal
 from itertools import groupby
 
 from sql import operators, Literal, Null
-from sql.conditionals import Coalesce
+from sql.conditionals import Coalesce, Case
 
 from trytond import backend
+from trytond.i18n import gettext
 from trytond.model import ModelSQL, ModelView, Workflow, fields, \
         sequence_ordered
+from trytond.model.exceptions import AccessError
 from trytond.pool import Pool
 from trytond.pyson import Eval, If, Bool
 from trytond.transaction import Transaction
+from trytond.tools import sortable_values
 from trytond.wizard import Wizard, StateView, StateAction, StateTransition, \
         Button
 
-from trytond.modules.product import price_digits
-
-__all__ = ['Subscription', 'Line', 'LineConsumption',
-    'CreateLineConsumption', 'CreateLineConsumptionStart',
-    'CreateSubscriptionInvoice', 'CreateSubscriptionInvoiceStart']
-STATES = [
-    ('draft', 'Draft'),
-    ('quotation', 'Quotation'),
-    ('running', 'Running'),
-    ('closed', 'Closed'),
-    ('canceled', 'Canceled'),
-    ]
+from trytond.modules.company.model import (
+    employee_field, set_employee, reset_employee)
+from trytond.modules.product import price_digits, round_price
+from .exceptions import InvalidRecurrence, InvoiceError
 
 
 class Subscription(Workflow, ModelSQL, ModelView):
@@ -38,13 +32,9 @@ class Subscription(Workflow, ModelSQL, ModelView):
     company = fields.Many2One(
         'company.company', "Company", required=True, select=True,
         states={
-            'readonly': Eval('state') != 'draft',
+            'readonly': (Eval('state') != 'draft') | Eval('party', True),
             },
-        domain=[
-            ('id', If(Eval('context', {}).contains('company'), '=', '!='),
-                Eval('context', {}).get('company', -1)),
-            ],
-        depends=['state'],
+        depends=['state', 'party'],
         help="Make the subscription belong to the company.")
 
     number = fields.Char(
@@ -66,18 +56,43 @@ class Subscription(Workflow, ModelSQL, ModelView):
             'readonly': ((Eval('state') != 'draft')
                 | (Eval('lines', [0]) & Eval('party'))),
             },
-        depends=['state'],
+        context={
+            'company': Eval('company', -1),
+            },
+        depends=['state', 'company'],
         help="The party who subscribes.")
+    contact = fields.Many2One(
+        'party.contact_mechanism', "Contact",
+        context={
+            'company': Eval('company', -1),
+            },
+        search_context={
+            'related_party': Eval('party'),
+            },
+        depends=['company', 'party'])
+    invoice_party = fields.Many2One('party.party', "Invoice Party",
+        states={
+            'readonly': ((Eval('state') != 'draft')
+                | Eval('lines', [0])),
+            },
+        context={
+            'company': Eval('company', -1),
+            },
+        search_context={
+            'related_party': Eval('party'),
+            },
+        depends=['state', 'company', 'party'])
     invoice_address = fields.Many2One(
         'party.address', "Invoice Address",
         domain=[
-            ('party', '=', Eval('party')),
+            ('party', '=', If(Bool(Eval('invoice_party',)),
+                    Eval('invoice_party'), Eval('party'))),
             ],
         states={
             'readonly': Eval('state') != 'draft',
             'required': ~Eval('state').in_(['draft']),
             },
-        depends=['party', 'state'])
+        depends=['party', 'invoice_party', 'state'])
     payment_term = fields.Many2One(
         'account.invoice.payment_term', "Payment Term",
         states={
@@ -137,8 +152,17 @@ class Subscription(Workflow, ModelSQL, ModelView):
             },
         depends=['state'])
 
-    state = fields.Selection(
-        STATES, "State", readonly=True, required=True,
+    quoted_by = employee_field(
+        "Quoted By", states=['quotation', 'running', 'closed', 'cancelled'])
+    run_by = employee_field(
+        "Run By", states=['running', 'closed', 'cancelled'])
+    state = fields.Selection([
+            ('draft', "Draft"),
+            ('quotation', "Quotation"),
+            ('running', "Running"),
+            ('closed', "Closed"),
+            ('cancelled', "Cancelled"),
+            ], "State", readonly=True, required=True,
         help="The current state of the subscription.")
 
     @classmethod
@@ -149,14 +173,14 @@ class Subscription(Workflow, ModelSQL, ModelView):
             ('id', 'DESC'),
             ]
         cls._transitions |= set((
-                ('draft', 'canceled'),
+                ('draft', 'cancelled'),
                 ('draft', 'quotation'),
-                ('quotation', 'canceled'),
+                ('quotation', 'cancelled'),
                 ('quotation', 'draft'),
                 ('quotation', 'running'),
                 ('running', 'draft'),
                 ('running', 'closed'),
-                ('canceled', 'draft'),
+                ('cancelled', 'draft'),
                 ))
         cls._buttons.update({
                 'cancel': {
@@ -166,7 +190,7 @@ class Subscription(Workflow, ModelSQL, ModelView):
                     },
                 'draft': {
                     'invisible': Eval('state').in_(['draft', 'closed']),
-                    'icon': If(Eval('state') == 'canceled',
+                    'icon': If(Eval('state') == 'cancelled',
                         'tryton-undo', 'tryton-back'),
                     'depends': ['state'],
                     },
@@ -183,6 +207,18 @@ class Subscription(Workflow, ModelSQL, ModelView):
                     },
                 })
 
+    @classmethod
+    def __register__(cls, module_name):
+        cursor = Transaction().connection.cursor()
+        table = cls.__table__()
+
+        super().__register__(module_name)
+
+        # Migration from 5.6: rename state canceled to cancelled
+        cursor.execute(*table.update(
+                [table.state], ['cancelled'],
+                where=table.state == 'canceled'))
+
     @classmethod
     def default_company(cls):
         return Transaction().context.get('company')
@@ -199,35 +235,61 @@ class Subscription(Workflow, ModelSQL, ModelView):
     def default_state(cls):
         return 'draft'
 
-    @fields.depends('party')
+    @fields.depends('party', 'invoice_party')
     def on_change_party(self):
-        self.invoice_address = None
+        if not self.invoice_party:
+            self.invoice_address = None
         if self.party:
-            self.invoice_address = self.party.address_get(type='invoice')
+            if not self.invoice_party:
+                self.invoice_address = self.party.address_get(type='invoice')
             self.payment_term = self.party.customer_payment_term
 
+    @fields.depends('party', 'invoice_party')
+    def on_change_invoice_party(self):
+        if self.invoice_party:
+            self.invoice_address = self.invoice_party.address_get(
+                type='invoice')
+        elif self.party:
+            self.invoice_address = self.party.address_get(type='invoice')
+
     @classmethod
     def set_number(cls, subscriptions):
         pool = Pool()
-        Sequence = pool.get('ir.sequence')
         Config = pool.get('sale.configuration')
 
         config = Config(1)
         for subscription in subscriptions:
             if subscription.number:
                 continue
-            subscription.number = Sequence.get_id(
-                config.subscription_sequence.id)
+            subscription.number = config.get_multivalue(
+                'subscription_sequence',
+                company=subscription.company.id).get()
         cls.save(subscriptions)
 
     def compute_next_invoice_date(self):
         start_date = self.invoice_start_date or self.start_date
-        date = self.next_invoice_date or self.start_date
+        date = self.next_invoice_date or start_date
         rruleset = self.invoice_recurrence.rruleset(start_date)
         dt = datetime.datetime.combine(date, datetime.time())
         inc = (start_date == date) and not self.next_invoice_date
         next_date = rruleset.after(dt, inc=inc)
-        return next_date.date()
+        if next_date:
+            return next_date.date()
+
+    @classmethod
+    def validate(cls, subscriptions):
+        super().validate(subscriptions)
+        for subscription in subscriptions:
+            subscription.validate_invoice_recurrence()
+
+    def validate_invoice_recurrence(self):
+        start_date = self.invoice_start_date or self.start_date
+        try:
+            self.invoice_recurrence.rruleset(start_date)[0]
+        except IndexError:
+            raise InvalidRecurrence(gettext(
+                    'sale_subscription.msg_invoice_recurrence_invalid',
+                    subscription=self.rec_name))
 
     @classmethod
     def copy(cls, subscriptions, default=None):
@@ -237,29 +299,41 @@ class Subscription(Workflow, ModelSQL, ModelView):
             default = default.copy()
         default.setdefault('number', None)
         default.setdefault('next_invoice_date', None)
+        default.setdefault('quoted_by')
+        default.setdefault('run_by')
         return super(Subscription, cls).copy(subscriptions, default=default)
 
+    @classmethod
+    def view_attributes(cls):
+        return super().view_attributes() + [
+            ('/tree', 'visual',
+                If(Eval('state') == 'cancelled', 'muted', '')),
+            ]
+
     @classmethod
     @ModelView.button
-    @Workflow.transition('canceled')
+    @Workflow.transition('cancelled')
     def cancel(cls, subscriptions):
         pass
 
     @classmethod
     @ModelView.button
     @Workflow.transition('draft')
+    @reset_employee('quoted_by', 'run_by')
     def draft(cls, subscriptions):
         pass
 
     @classmethod
     @ModelView.button
     @Workflow.transition('quotation')
+    @set_employee('quoted_by')
     def quote(cls, subscriptions):
         cls.set_number(subscriptions)
 
     @classmethod
     @ModelView.button
     @Workflow.transition('running')
+    @set_employee('run_by')
     def run(cls, subscriptions):
         pool = Pool()
         Line = pool.get('sale.subscription.line')
@@ -270,7 +344,7 @@ class Subscription(Workflow, ModelSQL, ModelView):
                     subscription.compute_next_invoice_date())
             for line in subscription.lines:
                 if (line.next_consumption_date is None
-                        and not line.consumed):
+                        and not line.consumed_until):
                     line.next_consumption_date = (
                         line.compute_next_consumption_date())
             lines.extend(subscription.lines)
@@ -289,7 +363,11 @@ class Subscription(Workflow, ModelSQL, ModelView):
     @classmethod
     @Workflow.transition('closed')
     def close(cls, subscriptions):
-        pass
+        for subscription in subscriptions:
+            if not subscription.end_date and subscription.lines:
+                subscription.end_date = max(
+                    l.end_date for l in subscription.lines)
+        cls.save(subscriptions)
 
     @classmethod
     def generate_invoice(cls, date=None):
@@ -306,6 +384,8 @@ class Subscription(Workflow, ModelSQL, ModelView):
                 ('invoice_line', '=', None),
                 ('line.subscription.next_invoice_date', '<=', date),
                 ('line.subscription.state', 'in', ['running', 'closed']),
+                ('line.subscription.company', '=',
+                    Transaction().context.get('company')),
                 ],
             order=[
                 ('line.subscription.id', 'DESC'),
@@ -357,13 +437,14 @@ class Subscription(Workflow, ModelSQL, ModelView):
     def _get_invoice(self):
         pool = Pool()
         Invoice = pool.get('account.invoice')
+        party = self.invoice_party or self.party
         invoice = Invoice(
             company=self.company,
             type='out',
-            party=self.party,
+            party=party,
             invoice_address=self.invoice_address,
             currency=self.currency,
-            account=self.party.account_receivable,
+            account=party.account_receivable_used,
             )
         invoice.on_change_type()
         invoice.payment_term = self.payment_term
@@ -384,7 +465,7 @@ class Line(sequence_ordered(), ModelSQL, ModelView):
         depends=['subscription_state'],
         help="Add the line below the subscription.")
     subscription_state = fields.Function(
-        fields.Selection(STATES, "Subscription State"),
+        fields.Selection('get_subscription_states', "Subscription State"),
         'on_change_with_subscription_state')
     subscription_start_date = fields.Function(
         fields.Date("Subscription Start Date"),
@@ -392,13 +473,19 @@ class Line(sequence_ordered(), ModelSQL, ModelView):
     subscription_end_date = fields.Function(
         fields.Date("Subscription End Date"),
         'on_change_with_subscription_end_date')
+    company = fields.Function(
+        fields.Many2One('company.company', "Company"),
+        'on_change_with_company')
 
     service = fields.Many2One(
         'sale.subscription.service', "Service", required=True,
         states={
             'readonly': Eval('subscription_state') != 'draft',
             },
-        depends=['subscription_state'])
+        context={
+            'company': Eval('company', None),
+            },
+        depends=['subscription_state', 'company'])
     description = fields.Text("Description",
         states={
             'readonly': Eval('subscription_state') != 'draft',
@@ -436,7 +523,8 @@ class Line(sequence_ordered(), ModelSQL, ModelView):
             'readonly': Eval('subscription_state') != 'draft',
             },
         depends=['subscription_state'])
-
+    currency = fields.Function(fields.Many2One('currency.currency',
+        'Currency'), 'on_change_with_currency')
     consumption_recurrence = fields.Many2One(
         'sale.subscription.recurrence.rule.set', "Consumption Recurrence",
         states={
@@ -454,7 +542,7 @@ class Line(sequence_ordered(), ModelSQL, ModelView):
     next_consumption_date_delayed = fields.Function(
         fields.Date("Next Consumption Delayed"),
         'get_next_consumption_date_delayed')
-    consumed = fields.Boolean("Consumed")
+    consumed_until = fields.Date("Consumed until", readonly=True)
     start_date = fields.Date(
         "Start Date", required=True,
         domain=[
@@ -462,9 +550,10 @@ class Line(sequence_ordered(), ModelSQL, ModelView):
             ],
         states={
             'readonly': ((Eval('subscription_state') != 'draft')
-                | Eval('consumed')),
+                | Eval('consumed_until')),
             },
-        depends=['subscription_start_date', 'subscription_state', 'consumed'])
+        depends=[
+            'subscription_start_date', 'subscription_state', 'consumed_until'])
     end_date = fields.Date(
         "End Date",
         domain=['OR', [
@@ -472,31 +561,35 @@ class Line(sequence_ordered(), ModelSQL, ModelView):
                 If(Bool(Eval('subscription_end_date')),
                     ('end_date', '<=', Eval('subscription_end_date')),
                     ()),
-                If(Bool(Eval('next_consumption_date')),
-                    ('end_date', '>=', Eval('next_consumption_date')),
+                If(Bool(Eval('consumed_until')),
+                    ('end_date', '>=', Eval('consumed_until')),
                     ()),
                 ],
             ('end_date', '=', None),
             ],
         states={
             'readonly': ((Eval('subscription_state') != 'draft')
-                | (~Eval('next_consumption_date') & Eval('consumed'))),
+                | (~Eval('consumed_until') & Eval('consumed_until'))),
             },
         depends=['subscription_end_date', 'start_date',
-            'next_consumption_date', 'subscription_state', 'consumed'])
+            'consumed_until', 'subscription_state', 'consumed_until'])
+
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        cls.__access__.add('subscription')
 
     @classmethod
     def __register__(cls, module):
         pool = Pool()
         Subscription = pool.get('sale.subscription')
-        TableHandler = backend.get('TableHandler')
         transaction = Transaction()
         cursor = transaction.connection.cursor()
         table = cls.__table__()
         subscription = Subscription.__table__()
 
         # Migration from 4.8: start_date required
-        if TableHandler.table_exist(cls._table):
+        if backend.TableHandler.table_exist(cls._table):
             table_h = cls.__table_handler__(module)
             if table_h.column_exist('start_date'):
                 cursor.execute(*table.update(
@@ -512,6 +605,21 @@ class Line(sequence_ordered(), ModelSQL, ModelView):
         # Migration from 4.8: drop required on description
         table_h.not_null_action('description', action='remove')
 
+        # Migration from 5.2: replace consumed by consumed_until
+        if table_h.column_exist('consumed'):
+            cursor.execute(*table.update(
+                    [table.consumed_until],
+                    [Case((table.consumed, Coalesce(
+                                    table.next_consumption_date,
+                                    table.end_date)), else_=Null)]))
+            table_h.drop_column('consumed')
+
+    @classmethod
+    def get_subscription_states(cls):
+        pool = Pool()
+        Subscription = pool.get('sale.subscription')
+        return Subscription.fields_get(['state'])['state']['selection']
+
     @fields.depends('subscription', '_parent_subscription.state')
     def on_change_with_subscription_state(self, name=None):
         if self.subscription:
@@ -527,6 +635,11 @@ class Line(sequence_ordered(), ModelSQL, ModelView):
         if self.subscription:
             return self.subscription.end_date
 
+    @fields.depends('subscription', '_parent_subscription.company')
+    def on_change_with_company(self, name=None):
+        if self.subscription and self.subscription.company:
+            return self.subscription.company.id
+
     @fields.depends('subscription', 'start_date', 'end_date',
         '_parent_subscription.start_date', '_parent_subscription.end_date')
     def on_change_subscription(self):
@@ -546,6 +659,11 @@ class Line(sequence_ordered(), ModelSQL, ModelView):
             return self.unit.digits
         return 2
 
+    @fields.depends('subscription', '_parent_subscription.currency')
+    def on_change_with_currency(self, name=None):
+        if self.subscription and self.subscription.currency:
+            return self.subscription.currency.id
+
     @fields.depends('service')
     def on_change_with_service_unit_category(self, name=None):
         if self.service:
@@ -580,14 +698,14 @@ class Line(sequence_ordered(), ModelSQL, ModelView):
             self.unit_price = Product.get_sale_price(
                 [product], self.quantity or 0)[product.id]
             if self.unit_price:
-                self.unit_price = self.unit_price.quantize(
-                    Decimal(1) / 10 ** self.__class__.unit_price.digits[1])
+                self.unit_price = round_price(self.unit_price)
 
         self.consumption_recurrence = self.service.consumption_recurrence
         self.consumption_delay = self.service.consumption_delay
 
     @fields.depends('subscription', '_parent_subscription.currency',
-        '_parent_subscription.party', 'start_date', 'unit', 'service')
+        '_parent_subscription.party', 'start_date', 'unit', 'service',
+        'company')
     def _get_context_sale_price(self):
         context = {}
         if self.subscription:
@@ -601,6 +719,8 @@ class Line(sequence_ordered(), ModelSQL, ModelView):
             context['uom'] = self.unit.id
         elif self.service:
             context['uom'] = self.service.sale_uom.id
+        if self.company:
+            context['company'] = self.company.id
         # TODO tax
         return context
 
@@ -609,12 +729,15 @@ class Line(sequence_ordered(), ModelSQL, ModelView):
             return self.next_consumption_date + self.consumption_delay
         return self.next_consumption_date
 
-    @classmethod
-    def default_consumed(cls):
-        return False
-
     def get_rec_name(self, name):
-        return '%s @ %s' % (self.service.rec_name, self.subscription.rec_name)
+        pool = Pool()
+        Lang = pool.get('ir.lang')
+        lang = Lang.get()
+        return (lang.format(
+                '%.*f', (self.unit.digits, self.quantity or 0))
+            + '%s %s @ %s' % (
+                self.unit.symbol, self.service.rec_name,
+                self.subscription.rec_name))
 
     @classmethod
     def search_rec_name(cls, name, clause):
@@ -655,6 +778,8 @@ class Line(sequence_ordered(), ModelSQL, ModelView):
                 ('consumption_recurrence', '!=', None),
                 ('next_consumption_date_delayed', '<=', date),
                 ('subscription.state', '=', 'running'),
+                ('subscription.company', '=',
+                    Transaction().context.get('company')),
                 ])
 
         consumptions = []
@@ -662,11 +787,16 @@ class Line(sequence_ordered(), ModelSQL, ModelView):
         while remainings:
             lines, remainings = remainings, []
             for line in lines:
-                consumptions.append(
-                    line.get_consumption(line.next_consumption_date))
+                consumption = line.get_consumption(line.next_consumption_date)
+                if consumption:
+                    consumptions.append(consumption)
                 line.next_consumption_date = (
                     line.compute_next_consumption_date())
-                line.consumed = True
+                if line.next_consumption_date:
+                    line.consumed_until = (
+                        line.next_consumption_date - datetime.timedelta(1))
+                else:
+                    line.consumed_until = line.end_date
                 if line.next_consumption_date is None:
                     subscription_ids.add(line.subscription.id)
                 elif line.get_next_consumption_date_delayed() <= date:
@@ -679,7 +809,9 @@ class Line(sequence_ordered(), ModelSQL, ModelView):
     def get_consumption(self, date):
         pool = Pool()
         Consumption = pool.get('sale.subscription.line.consumption')
-        return Consumption(line=self, quantity=self.quantity, date=date)
+        end_date = self.end_date or self.subscription.end_date
+        if date < (end_date or datetime.date.max):
+            return Consumption(line=self, quantity=self.quantity, date=date)
 
     def compute_next_consumption_date(self):
         if not self.consumption_recurrence:
@@ -688,12 +820,30 @@ class Line(sequence_ordered(), ModelSQL, ModelView):
         rruleset = self.consumption_recurrence.rruleset(self.start_date)
         dt = datetime.datetime.combine(date, datetime.time())
         inc = (self.start_date == date) and not self.next_consumption_date
-        next_date = rruleset.after(dt, inc=inc).date()
-        for end_date in [self.end_date, self.subscription.end_date]:
-            if end_date:
-                if next_date > end_date:
-                    return None
-        return next_date
+        next_date = rruleset.after(dt, inc=inc)
+        if next_date:
+            next_date = next_date.date()
+            for end_date in [self.end_date, self.subscription.end_date]:
+                if end_date:
+                    if next_date > end_date:
+                        return None
+            return next_date
+
+    @classmethod
+    def validate(cls, lines):
+        super().validate(lines)
+        for line in lines:
+            line.validate_consumption_recurrence()
+
+    def validate_consumption_recurrence(self):
+        if self.consumption_recurrence:
+            try:
+                self.consumption_recurrence.rruleset(self.start_date)[0]
+            except IndexError:
+                raise InvalidRecurrence(gettext(
+                        'sale_subscription.msg_consumption_recurrence_invalid',
+                        line=self.rec_name,
+                        subscription=self.subscription.rec_name))
 
     @classmethod
     def copy(cls, lines, default=None):
@@ -715,7 +865,7 @@ class LineConsumption(ModelSQL, ModelView):
         'sale.subscription.line', "Line", required=True, select=True,
         ondelete='RESTRICT')
     quantity = fields.Float(
-        "Quantity", digits=(16, Eval('unit_digits', 2)),
+        "Quantity", digits=(16, Eval('unit_digits', 2)), required=True,
         depends=['unit_digits'])
     unit_digits = fields.Function(
         fields.Integer("Unit Digits"), 'on_change_with_unit_digits')
@@ -726,15 +876,8 @@ class LineConsumption(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(LineConsumption, cls).__setup__()
+        cls.__access__.add('line')
         cls._order.insert(0, ('date', 'DESC'))
-        cls._error_messages.update({
-                'modify_invoiced_consumption': (
-                    "You can not modify invoiced consumption."),
-                'delete_invoiced_consumption': (
-                    "You can not delete invoiced consumption."),
-                'missing_account_revenue': ('Product "%(product)s" '
-                    'misses a revenue account.'),
-                })
 
     @fields.depends('line')
     def on_change_with_unit_digits(self, name=None):
@@ -753,16 +896,23 @@ class LineConsumption(ModelSQL, ModelView):
 
     @classmethod
     def write(cls, *args):
-        if any(c.invoice_line
-                for consumptions in args[::2]
-                for c in consumptions):
-            cls.raise_user_error('modify_invoiced_consumption')
+        for consumptions in args[::2]:
+            for consumption in consumptions:
+                if consumption.invoice_line:
+                    raise AccessError(
+                        gettext('sale_subscription'
+                            '.msg_consumption_modify_invoiced',
+                            consumption=consumption.rec_name))
         super(LineConsumption, cls).write(*args)
 
     @classmethod
     def delete(cls, consumptions):
-        if any(c.invoice_line for c in consumptions):
-            cls.raise_user_error('delete_invoiced_consumption')
+        for consumption in consumptions:
+            if consumption.invoice_line:
+                raise AccessError(
+                    gettext('sale_subscription'
+                        '.msg_consumption_modify_invoiced',
+                        consumption=consumption.rec_name))
         super(LineConsumption, cls).delete(consumptions)
 
     @classmethod
@@ -772,7 +922,8 @@ class LineConsumption(ModelSQL, ModelView):
         InvoiceLine = pool.get('account.invoice.line')
 
         lines, grouped_consumptions = [], []
-        consumptions = sorted(consumptions, key=cls._group_invoice_key)
+        consumptions = sorted(
+            consumptions, key=sortable_values(cls._group_invoice_key))
         for key, sub_consumptions in groupby(
                 consumptions, key=cls._group_invoice_key):
             sub_consumptions = list(sub_consumptions)
@@ -783,9 +934,10 @@ class LineConsumption(ModelSQL, ModelView):
 
             line.account = line.product.account_revenue_used
             if not line.account:
-                cls.raise_user_error('missing_account_revenue', {
-                        'product': line.product.rec_name,
-                        })
+                raise InvoiceError(
+                    gettext('sale_subscription'
+                        '.msg_consumption_invoice_missing_account_revenue',
+                        product=line.product.rec_name))
 
             taxes = []
             pattern = line._get_tax_rule_pattern()
@@ -811,6 +963,7 @@ class LineConsumption(ModelSQL, ModelView):
     def _group_invoice_key(cls, consumption):
         return (
             ('company', consumption.line.subscription.company),
+            ('currency', consumption.line.subscription.currency),
             ('unit', consumption.line.unit),
             ('product', consumption.line.service.product),
             ('unit_price', consumption.line.unit_price),
